import uuid

from sqlalchemy import (
    Column,
    Integer,
    Text,
    ForeignKey,
    Enum,
    Float,
    Date,
    Boolean,
    MetaData,
)
from sqlalchemy.orm import relationship, declarative_base

from constants.reddit import ClassificationType

metadata = MetaData()
Base = declarative_base(metadata=metadata)


class Post(Base):
    __tablename__ = "posts"

    id = Column(Text, primary_key=True)
    title = Column(Text, nullable=False)
    url = Column(Text, nullable=False)
    score = Column(Integer, nullable=False)
    num_comments = Column(Integer, nullable=False)
    country = Column(Text, nullable=False, default="unknown")


class Comment(Base):
    __tablename__ = "comments"

    id = Column(Text, primary_key=True)
    post_id = Column(Text, ForeignKey("posts.id"), nullable=False)
    body = Column(Text, nullable=False)
    score = Column(Integer, nullable=False)
    classification = Column(Enum(ClassificationType), nullable=False)
    start_date = Column(Date, nullable=True)
    end_date = Column(Date, nullable=True)
    characteristic = Column(Text, nullable=False)
    summary = Column(Text, nullable=False)

    locations = relationship(
        "Location", backref="comment", cascade="all, delete-orphan"
    )

    post = relationship("Post", backref="comments")


class Location(Base):
    __tablename__ = "locations"

    id = Column(Text, primary_key=True, default=lambda: str(uuid.uuid4()))
    comment_id = Column(Text, ForeignKey("comments.id", ondelete="CASCADE"))
    lat = Column(Float, nullable=False)
    lng = Column(Float, nullable=False)
    location_name = Column(Text, nullable=False)
    characteristic = Column(Text, nullable=False)
    off_the_beaten = Column(Boolean, nullable=False)



import enum

SUBREDDIT_COUNTRY_MAPPING = {
    "japantravel": "japan",
    "koreatravel": "korea",
}


class ClassificationType(enum.Enum):
    travel_tip = "travel_tip"
    travel_suggestion = "travel_suggestion"
    other = "other"



from typing import Any

from sqlalchemy.orm import Session
from sqlalchemy.exc import SQLAlchemyError


class Repository:
    def __init__(self, session: Session):
        self.session = session

    def get_all(self, model) -> list:
        """Generic method to fetch all records from a given model."""
        records = self.session.query(model).all()
        return records

    def filter(self, model, **filters) -> list:
        """Generic method to filter records from a given model."""
        query = self.session.query(model)
        for key, value in filters.items():
            query = query.filter(getattr(model, key) == value)
        return query.all()

    def get(self, model, record_id) -> Any:
        """Get a single record by ID."""
        record = self.session.get(model, record_id)
        return record

    def commit(self):
        """Commit the current transaction."""
        self.session.commit()

    def rollback(self):
        """Rollback the current transaction."""
        self.session.rollback()

    def add(self, model, **data) -> dict[str, Any]:
        """Insert a new row into the given model."""
        try:
            instance = model(**data)
            self.session.add(instance)
            self.commit()
            return instance
        except SQLAlchemyError as e:
            self.rollback()
            raise e

    def update(self, model, record_id, **data) -> Any:
        """Update an existing row by ID."""
        try:
            instance = self.get(model, record_id)
            if instance:
                for key, value in data.items():
                    setattr(instance, key, value)
                self.commit()
                return instance
            else:
                return None
        except SQLAlchemyError as e:
            self.rollback()
            raise e

    def delete(self, model, record_id) -> bool:
        """Delete a row by ID."""
        try:
            instance = self.get(model, record_id)
            if instance:
                self.session.delete(instance)
                self.commit()
                return True
            return False
        except SQLAlchemyError as e:
            self.rollback()
            raise e



from abc import ABC, abstractmethod
from typing import IO


class BaseFSAccess(ABC):
    @abstractmethod
    def get_file_paths(self, directory: str, file_type: str) -> list[str]:
        pass

    @abstractmethod
    def open(self, path: str, mode: str) -> IO:
        # open the given file, mode = "r"/"w"
        pass



import glob
import os
from contextlib import contextmanager
from typing import IO

from fs_access.base_fs_access import BaseFSAccess


class LocalFSAccess(BaseFSAccess):
    def get_file_paths(self, directory: str, file_type: str) -> list[str]:
        return glob.glob(
            os.path.join(directory, "**", f"*.{file_type.lower()}"),
            recursive=True,
        )

    @contextmanager
    def open(self, path: str, mode: str = "r") -> IO:
        with open(path, mode) as f:
            yield f
